/*#######################################################################################
SPI_LCD with HD44780 LCD controller  

Copyright (C) 2005 Branko Golubovic
branko.golubovic@gmx.at
#######################################################################################*/


#include "main.h"
#include "spi_lcd.h"

static unsigned char base_y[4]={0x00,0x40,0x00+LCD_COLUMNS,0x40+LCD_COLUMNS};
unsigned char lcd_x,lcd_y,lcd_xmax,leds;


//local functions

/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
static inline void _delayFourCycles(unsigned int __count)
{
    if ( __count == 0 )    
        __asm__ __volatile__( "rjmp 1f\n 1:" );    // 2 cycles
    else
        __asm__ __volatile__ (
    	    "1: sbiw %0,1" "\n\t"                  
    	    "brne 1b"                              // 4 cycles/loop
    	    : "=w" (__count)
    	    : "0" (__count)
    	   );
}

/************************************************************************* 
delay for a minimum of <us> microseconds
the number of loops is calculated at compile-time from MCU clock frequency
*************************************************************************/
#define delay(us)  _delayFourCycles( ( ( 1*(SYSCLK/4000) )*us)/1000 )


//############################################################################
// Generate E Pulse
 void spi_lcd_E(void)
//############################################################################
{
	#if HARDWARE_SPI
	spi_disable();
	#endif
	SPI_Write&=~(1<<SPI_Clock);				//Clock Low, SS High-> E High
	delay(1);														// Delay 
	SPI_Write|=(1<<SPI_Clock);					//Clock High,SS High-> E Low

	#if HARDWARE_SPI
	spi_enable();
	#endif
}


//############################################################################
// Send LCD controller instruction init  command
static inline void spi_lcd_init_command(unsigned char command)
//############################################################################
{
	SPI_SS_low();									//
	spi_write((0<<RS)|command);	// Write Command
	SPI_SS_high();								// Command to LCD
	spi_lcd_E();									// E pulse

	
	
}

// PuBLUC FUNCTIONS

//############################################################################
// Send LCD controller instruction command
void spi_lcd_command(unsigned char command)
//############################################################################
{
	SPI_SS_low();									//
	spi_write((0<<RS)|leds|(command & 0xf0));	// Write High Command Nible 
	SPI_SS_high();								// Command to LCD
	spi_lcd_E();									// Generate E pulse
	SPI_SS_low();									//
	spi_write((0<<RS)|leds|(command*16));	// Write Low Command Nible 
	SPI_SS_high();								// Command to LCD
	spi_lcd_E();									// Generate E pulse
	delay(40);
}

//############################################################################
// LCD Backlight On/Off
void spi_lcd_leds(unsigned char c)
//############################################################################
{
	SPI_SS_low();
	spi_write(c);
	SPI_SS_high();
	
}

//############################################################################
//write a byte to the LCD character generator or display RAM
void spi_lcd_putchar(unsigned char c)
//############################################################################
{	
	++lcd_x;
	if(lcd_x>lcd_xmax)
	{
		++lcd_y;
		spi_lcd_gotoxy(0,lcd_y);
		
	}
	SPI_SS_low();									//
	spi_write((1<<RS)|leds|(c & 0xf0));	// Write High Data Nible 
	SPI_SS_high();								// Command to LCD
	spi_lcd_E();									// Generate E pulse
	SPI_SS_low();									//
	spi_write((1<<RS)|leds|(c*16));	// Write Low Data Nible 
	SPI_SS_high();								// Command to LCD
	spi_lcd_E();
	delay(40);
}


//############################################################################
//Initialize the LCD controller
void spi_lcd_init(unsigned char lcd_columns)
//############################################################################
{
 lcd_xmax=lcd_columns;

 delay(16000);        // wait 16ms or more after power-on 
 spi_lcd_init_command(LCD_FUNCTION_8BIT_1LINE);	// LCD Function 8bit
 delay(5000);		// delay, busy flag can't be checked here
 spi_lcd_init_command(LCD_FUNCTION_8BIT_1LINE);	// LCD Function 8bit
 delay(100);		// delay, busy flag can't be checked here
 spi_lcd_init_command(LCD_FUNCTION_8BIT_1LINE);	// LCD Function 8bit
 delay(100);		// delay, busy flag can't be checked here
 spi_lcd_init_command(LCD_FUNCTION_4BIT_1LINE); // LCD Function 4bit
 delay(100);
 
 spi_lcd_command(LCD_FUNCTION_4BIT_2LINES);		// LCD function 4bit,Dual Line  
 delay(100);
 spi_lcd_command(LCD_DISP_OFF);				// Display Off
 delay(100);
 spi_lcd_command((1<<LCD_CLR)|leds);	// Clear LCD
 delay(2000);
 spi_lcd_command(LCD_MODE_DEFAULT);   //Set entry mode               
 delay(100);
 spi_lcd_command(LCD_DISP_ON);				// Display on
 delay(100);
 spi_lcd_command(1<<LCD_HOME);				// Cursor Home
 delay(2000);				
}


//############################################################################
/// Set the LCD display position  x=0..19 y=0..3
void spi_lcd_gotoxy(unsigned char x, unsigned char y)
//############################################################################
{
	spi_lcd_command((1<<LCD_DDRAM)|(base_y[y]+x));
	lcd_x=x;lcd_y=y;
	
}

//############################################################################
// Clear the LCD
void spi_lcd_clear(void)
//############################################################################
{
    spi_lcd_command((1<<LCD_CLR)|leds);
}

//############################################################################
// Clear the LCD
void spi_lcd_home(void)
//############################################################################
{
    spi_lcd_command((1<<LCD_HOME)|leds);
		lcd_x=0;lcd_y=0;
}




/*************************************************************************
Display string without auto linefeed 
Input:    string to be displayed
Returns:  none
*************************************************************************/
void spi_lcd_puts(const char *s)
/* print string on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = *s++) ) {
        spi_lcd_putchar(c);
    }

}/* lcd_puts */


/*************************************************************************
Display string from program memory without auto linefeed 
Input:     string from program memory be be displayed                                        
Returns:   none
*************************************************************************/
void spi_lcd_puts_p(const char *progmem_s)
/* print string from program memory on lcd (no auto linefeed) */
{
    register char c;

    while ( (c = pgm_read_byte(progmem_s++)) ) {
        spi_lcd_putchar(c);
    }

}/* lcd_puts_p */


//############################################################################
// Read the LCD Tastatur
unsigned char spi_lcd_keys(void)
//############################################################################
{ 
	unsigned char keys;
		SPI_SS_low();
		keys=spi_read_write(leds);
		SPI_SS_high();
	
	return keys^=0xff;
}
