/*#######################################################################################
SPI_LCD with HD44780 LCD controller 

Copyright (C) 2005 Branko Golubovic
branko.golubovic@gmx.at
#######################################################################################*/

#ifndef _SPI_LCD_H
 #define _SPI_LCD_H
 
#define RS										0			// RS Pin Position= Q0(HC595)
#define BG_LIGHT							1			// Background Light Pin Position= Q1(HC595)
#define LED1									2			// LED1 Pin Position= Q2(HC595)
#define LED2									3			// LED2 Pin Position= Q3(HC595)
#define LCD_COLUMNS						16		// 
 
#define LCD_CLR               0      // DB0: clear display  
#define LCD_HOME              1      // DB1: return to home position
#define LCD_ENTRY_MODE        2      // DB2: set entry mode
#define LCD_ENTRY_INC         1      //   DB1: 1=increment, 0=decrement 
#define LCD_ON_DISPLAY        2      //   DB2: turn display on
#define LCD_FUNCTION          5      // DB5: function set                   
#define LCD_FUNCTION_8BIT     4      //   DB4: set 8BIT mode (0->4BIT mode) 
#define LCD_FUNCTION_2LINES   3      //   DB3: two lines (0->one line)
#define LCD_CGRAM             6      // DB6: set CG RAM address             
#define LCD_DDRAM             7      // DB7: set DD RAM address


/* set entry mode: display shift on/off, dec/inc cursor move direction */
#define LCD_ENTRY_DEC            0x04   /* display shift off, dec cursor move dir */
#define LCD_ENTRY_DEC_SHIFT      0x05   /* display shift on,  dec cursor move dir */
//#define LCD_ENTRY_INC_           0x06   /* display shift off, inc cursor move dir */
#define LCD_ENTRY_INC_SHIFT      0x07   /* display shift on,  inc cursor move dir */

/* display on/off, cursor on/off, blinking char at cursor position */
#define LCD_DISP_OFF             0x08   /* display off                            */
#define LCD_DISP_ON              0x0C   /* display on, cursor off                 */
#define LCD_DISP_ON_BLINK        0x0D   /* display on, cursor off, blink char     */
#define LCD_DISP_ON_CURSOR       0x0E   /* display on, cursor on                  */
#define LCD_DISP_ON_CURSOR_BLINK 0x0F   /* display on, cursor on, blink char      */

/* move cursor/shift display */
#define LCD_MOVE_CURSOR_LEFT     0x10   /* move cursor left  (decrement)          */
#define LCD_MOVE_CURSOR_RIGHT    0x14   /* move cursor right (increment)          */
#define LCD_MOVE_DISP_LEFT       0x18   /* shift display left                     */
#define LCD_MOVE_DISP_RIGHT      0x1C   /* shift display right                    */

/* function set: set interface data length and number of display lines */
#define LCD_FUNCTION_4BIT_1LINE  0x20   /* 4-bit interface, single line, 5x7 dots */
#define LCD_FUNCTION_4BIT_2LINES 0x28   /* 4-bit interface, dual line,   5x7 dots */
#define LCD_FUNCTION_8BIT_1LINE  0x30   /* 8-bit interface, single line, 5x7 dots */
#define LCD_FUNCTION_8BIT_2LINES 0x38   /* 8-bit interface, dual line,   5x7 dots */

#define LCD_MODE_DEFAULT     ((1<<LCD_ENTRY_MODE) | (1<<LCD_ENTRY_INC) )

extern void spi_lcd_command(unsigned char command);
extern void spi_lcd_putchar(unsigned char c);
extern void spi_lcd_init(unsigned char lcd_columns);
extern void spi_lcd_gotoxy(unsigned char x, unsigned char y);
extern void spi_lcd_clear(void);
extern void spi_lcd_home(void);
extern void spi_lcd_puts(const char *s);
extern void spi_lcd_puts_p(const char *progmem_s);
extern void spi_lcd_leds(unsigned char c);
extern unsigned char spi_lcd_keys(void);

extern unsigned char leds;


 #endif // _SPI_LCD_H